unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Stringz, ExtCtrls, ShellAPI, ComCtrls;

type
  TLines = Array [ 1 .. 1024 ] Of String;
  TRule = Record
    nLines : Integer;
    Name : String;
    Lines : TLines;
  End;

	TFormMain = class(TForm)
		cmdConvert:        TButton;
		cmdBrowse:         TButton;
		cboCharacterSet:   TComboBox;
		chkScreen:         TCheckBox;
		chkInsertCR:       TCheckBox;
		EditFilename:      TEdit;
		EditCRChars:       TEdit;
		LabelFilename:     TLabel;
		LabelInsertCR:     TLabel;
		LabelCharacterSet: TLabel;
		LabelOtherOptions: TLabel;
		OpenDialog1:       TOpenDialog;
		prog:              TProgressBar;
		Separator:         TBevel;

		procedure cmdConvertClick(Sender: TObject);
		procedure cmdBrowseClick(Sender: TObject);
		procedure FormCreate(Sender: TObject);
		procedure FormActivate(Sender: TObject);
		procedure chkInsertCRClick(Sender: TObject);
		procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    { Private declarations }
    nRules : Integer;
    iRule : Integer;
    Rules : Array [ 1 .. 1024 ] Of TRule;
    function ExecuteFile(const FileName, Params, DefaultDir: string;
    	ShowCmd: Integer): THandle;
    Procedure ReadRules;
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

{$R *.DFM}

(**
 * Runs an executable program.
 *)
function TFormMain.ExecuteFile(const FileName, Params, DefaultDir: string;
	ShowCmd: Integer): THandle;
Var
	zFileName, zParams, zDir: array[0..79] of Char;
Begin
	Result := ShellExecute(Application.MainForm.Handle, nil,
		StrPCopy(zFileName, FileName), StrPCopy(zParams, Params),
		StrPCopy(zDir, DefaultDir), ShowCmd);
end;

procedure TFormMain.cmdConvertClick(Sender: TObject);
var
	b : Byte;
	f : File;
	tf : TextFile;
	max : Integer;
	data : Array [ 0 .. $f0000 ] Of Byte;
	i, cnt, p, r : Integer;
  s1, s1b, s2 : String;
  bMatch : Boolean;
  //
  LineWidth : Integer;
begin
	iRule := cboCharacterSet.ItemIndex + 1;
	ReadRules;
	//
	If EditFilename.Text = '' Then
	Begin
		ShowMessage ( 'Please select a file to convert.' );
	End
	Else
	Begin
		AssignFile ( f, EditFilename.Text );
		Reset ( f, 1 );
		//
		AssignFile ( tf, EditFilename.Text + '.txt' );
		ReWrite ( tf );
		//
		max := Filesize ( f );
		BlockRead ( f, data, max );
		CloseFile ( f );
		//
		LineWidth := SafeVal ( EditCRChars.Text );
		//
		prog.Max := max;
		//
		cnt := 0;
		i := 0;
		//
		If UpperCase ( KeepFromRight ( EditFilename.Text, 3 ) ) = 'PRG' Then
		begin
			// skip loading address
			i := 2;
		End;
		//
		While i <= max Do
		Begin
			b := data [ i ];
			//
			For r := 1 To Rules [ iRule ].nLines Do
			Begin
				// read one rule and separate it at the equals sign
				s1 := Rules [ iRule ].Lines [ r ];
				s2 := RemoveFromLeft ( s1, Pos ( '=', s1 ) );
				s1 := Copy ( s1, 1, Pos ( '=', s1 ) - 1 );
				//
				// match range...
				If Pos ( '..', s1 ) > 0 Then
				Begin
					s1b := RemoveFromLeft ( s1, Pos ( '..', s1 ) + 1 );
					s1 := Copy ( s1, 1, Pos ( '..', s1 ) - 1 );
					//
					bMatch := ( b >= SafeVal ( s1 ) ) And ( b <= SafeVal ( s1b ) );
				End
				Else
				Begin
					// or match single byte
					If Length ( s1 ) = 1 Then
					Begin
						bMatch := b = Ord ( s1 [ 1 ] );
					End
					Else
					Begin
						bMatch := b = SafeVal ( s1 );
					End;
				End;
				//
				If bMatch Then
				Begin
					//
					// this is a match, do the conversion
					If Length ( s2 ) = 1 Then
					Begin
						b := Ord ( s2 [ 1 ] );
					End
					Else
					Begin
						If s2 [ 1 ] = '+' Then
						Begin
							b := b + SafeVal ( RemoveFromLeft ( s2, 1 ) );
						End
						Else
						Begin
							If s2 [ 1 ] = '-' Then
							Begin
								b := b - SafeVal ( RemoveFromLeft ( s2, 1 ) );
							End
							Else
							Begin
								If s2 [ 1 ] = '/' Then
								Begin
									b := b Div SafeVal ( RemoveFromLeft ( s2, 1 ) );
								End
								Else
								Begin
									If s2 [ 1 ] = '*' Then
									Begin
										b := b * SafeVal ( RemoveFromLeft ( s2, 1 ) );
									End
									Else
									Begin
										While s2 <> '' Do
										Begin
											p := Pos ( ',', s2 );
											If p = 0 Then p := Length ( s2 ) + 1;
											b := SafeVal ( Copy ( s2, 1, p - 1 ) );
											s2 := RemoveFromLeft ( s2, p );
											//
											If s2 <> '' Then
												Write ( tf, Chr ( b ) );
										End;
									End;
								End;
							End;
						End;
					End;
				End;
			End;

			Write ( tf, Chr ( b ) );
			{
			If ( b < $60 ) And ( b <> $20 ) Then
				Write ( tf, ' ' + IntToHex( b, 2 ) + ' ' );
			}

			Inc ( cnt );
			If ( chkInsertCR.Checked ) And ( LineWidth <> 0 ) Then
			Begin
				If ( cnt Mod LineWidth ) = 0 Then
				Begin
					WriteLn ( tf );
				End;
				If chkScreen.Checked Then
				Begin
					If cnt = 1024 Then
					Begin
						WriteLn ( tf );
						cnt := 0;
					End;
				End;
			End;
			//
			If ( i And 15 ) = 0 Then
			Begin
				prog.StepBy ( 16 );
				Application.ProcessMessages;
			End;
			//
			Inc ( i );
		End;
		CloseFile ( tf );
		//
		prog.Max := 0;
		//
		//ExecuteFile ( 'c:\windows\notepad.exe', EditFilename.Text + '.txt', '', SW_NORMAL );
		ExecuteFile ( EditFilename.Text + '.txt', '', '', SW_NORMAL );
	End;
end;

procedure TFormMain.cmdBrowseClick(Sender: TObject);
begin
  If OpenDialog1.Execute Then
  Begin
    EditFilename.Text := OpenDialog1.FileName;
    cmdConvert.SetFocus;
  End;
end;

procedure TFormMain.FormCreate(Sender: TObject);
Var
	i : Integer;
	s : String;
begin
	If ParamCount > 0 Then
	Begin
		EditFilename.Text := ParamStr ( 1 );
	End;
	//
	s := ParamStr ( 0 );
	s := Copy ( s, 1, PosR ( '\', s ) );
	Application.HelpFile := s + 'p4text.hlp';
  //
  ReadRules;
  //
  For i := 1 To nRules Do
  Begin
    cboCharacterSet.Items.Add ( Rules [ i ].Name );
  End;
end;

Procedure TFormMain.ReadRules;
Var
  tf : TextFile;
  s : String;
Begin
  s := ParamStr ( 0 );
  s := Copy ( s, 1, PosR ( '\', s ) ) + 'rules.txt';
  If Not FileExists ( s ) Then
  Begin
    ShowMessage ( 'Rules definition file not found.' + #13 + s );
  End
  Else
  Begin
    AssignFile ( tf, s );
    Reset ( tf );
    //
    nRules := 0;
    //
    While Not Eof ( tf ) Do
    Begin
			ReadLn ( tf, s );
			// check for comments
			//
			If Length ( s ) <> 0 Then
			Begin
				// if the first char is ;, ignore the entire line
				if ( s [ 1 ] <> ';' ) Then
				Begin
					if ( Pos( ' ;', s ) > 0 ) Then
					Begin
						s := Copy( s, 1, Pos( ' ;', s ) ); // remove comments
					End;

					// look for [ to find beginning of a new rule set
					If s [ 1 ] = '[' Then
					Begin
						Inc ( nRules );
						Rules [ nRules ].nLines := 0;
						Rules [ nRules ].Name := s;
					End
					Else
					Begin
						Inc ( Rules [ nRules ].nLines );
						Rules [ nRules ].Lines [ Rules [ nRules ].nLines ] := s;
					End;
				End;
      End;
    End;
    CloseFile ( tf );
  End;
End;

procedure TFormMain.FormActivate(Sender: TObject);
begin
	cboCharacterSet.ItemIndex := 0;
end;

procedure TFormMain.chkInsertCRClick(Sender: TObject);
begin
	EditCRChars.Enabled := chkInsertCR.Checked;
	chkScreen.Enabled := chkInsertCR.Checked;
end;

procedure TFormMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
	If Key = 116 Then
	Begin
		// "hidden" feature: refresh rules file when F5 is pressed.
		ReadRules;
		Caption := 'Rules refreshed. (' + IntToStr ( nRules ) + ')'; 
	End;
end;

end.
